package Site::CrashNotify;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use strict;
use vars qw( $NOTIFY_TO $NOTIFY_FROM $NOTIFY_SUBJECT $SAVE_SIGDIE );

use CGI;
use CGI::Carp;
use Carp;
use Mail::Sender;

$NOTIFY_TO = 'root@localhost';
$NOTIFY_FROM = 'site_tools@localhost';
$NOTIFY_SUBJECT = 'Website error: sigdie';

sub import
{
	shift();
	my %opts = @_;

	$NOTIFY_TO = $opts{ to } || $NOTIFY_TO;
	$NOTIFY_SUBJECT = $opts{ subject } || $NOTIFY_SUBJECT;
	$NOTIFY_FROM = $opts{ from } || $NOTIFY_FROM;

	$SAVE_SIGDIE = $SIG{ __DIE__ };
	$SIG{ __DIE__ } = \&SIGDIE;
	return(1);
}

sub SIGDIE
{
	my $err = shift;
	my $q = new CGI;
	my $opt = $q -> Vars();

	my $mailer = new Mail::Sender { 
		from => $NOTIFY_FROM,
		smtp => 'localhost'
	};

	$mailer -> Open( {
		to => $NOTIFY_TO,
		subject => $NOTIFY_SUBJECT
	} );

	$mailer -> Send( "Died with error:\n$err" );
	$mailer -> Send( "\n\nScript Args " . '-' x 45 . "\n" );
        for( keys( %{ $opt } ) )
        {
                $mailer -> Send( $_ . ": " . $opt -> { $_ } . "\n" );
        }

        $mailer -> Send( "\n\nEnvironment " . '-' x 45 . "\n" );
        for( keys( %::ENV ) )
        {
                $mailer -> Send( $_ . ": " . $::ENV{ $_ } . "\n" );
        }

	$mailer -> Close();

	# Note for browser?
	print qq{Content-type: text/html

<BR><BR>
<H1>There has been an internal server error.</H1>
<BR>
The actual error was: $err
<BR><BR>
The website/server administrator has been emailed regarding this problem.
If this problem persists, please contact the server admin at
<a href="mailto:$::ENV{SERVER_ADMIN}">$::ENV{SERVER_ADMIN}</a>.
<HR>
$::ENV{SERVER_SIGNATURE}
<BR><BR>
};

	if( $SAVE_SIGDIE )
	{
		#&{ $SAVE_SIGDIE };
	}

	Carp::longmess( "Died" );
}

1;
 

__END__

=head1 NAME

Site::CrashNotify - Send email dumping environment, error, and page args
when an error occurs (internal server error)

=head1 SYNOPSIS

  use Site::CrashNotify ( to => 'you@yourhost.com', subject => 'It broke!' );

=head1 DESCRIPTION

Just 'use' the module and pass at least a 'to' argument in the use list,
and a SIGDIE handler will be put in place to send you a notification
when something goes wrong.  An easy way to test and get a sample message
would be like this:

  use Site::CrashNotify ( to => 'you@yourhost.com' );

  print 'fooo'
  print 'jub'

The line terminators are intentionally missing.  The script will crash and
you should get an email summary at 'you@yourhost.com'.

It you do not specify a to address, then CrashNotify will die() immediately.

=head1 AUTHOR

Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.

This program is free software  and is provided "as is" without
express or implied warranty. You can redistribute it and/or
modify it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1).

=cut
